package cn.com.infosec.x509;

import cn.com.infosec.asn1.ASN1EncodableVector;
import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.DERBitString;
import cn.com.infosec.asn1.DEREncodable;
import cn.com.infosec.asn1.DERInteger;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.DERSequence;
import cn.com.infosec.asn1.x509.AlgorithmIdentifier;
import cn.com.infosec.asn1.x509.SubjectPublicKeyInfo;
import cn.com.infosec.asn1.x509.TBSCertificateStructure;
import cn.com.infosec.asn1.x509.Time;
import cn.com.infosec.asn1.x509.V3TBSCertificateGenerator;
import cn.com.infosec.asn1.x509.X509CertificateStructure;
import cn.com.infosec.asn1.x509.X509ExtensionsGenerator;
import cn.com.infosec.asn1.x509.X509Name;
import cn.com.infosec.jce.X509Principal;
import cn.com.infosec.jce.provider.X509CertificateObject;
import cn.com.infosec.x509.extension.X509ExtensionUtil;
import java.io.IOException;
import java.math.BigInteger;
import java.security.GeneralSecurityException;
import java.security.InvalidKeyException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.SignatureException;
import java.security.cert.CertificateParsingException;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.Iterator;
import javax.security.auth.x500.X500Principal;

public class X509V3CertificateGenerator {
   private V3TBSCertificateGenerator tbsGen;
   private DERObjectIdentifier sigOID;
   private AlgorithmIdentifier sigAlgId;
   private String signatureAlgorithm;
   private X509ExtensionsGenerator extGenerator;

   public X509V3CertificateGenerator() {
      X509V3CertificateGenerator var10000 = this;
      X509V3CertificateGenerator var10001 = this;
      super();
      V3TBSCertificateGenerator var1;
      var1 = new V3TBSCertificateGenerator.<init>();
      var10001.tbsGen = var1;
      X509ExtensionsGenerator var2;
      var2 = new X509ExtensionsGenerator.<init>();
      var10000.extGenerator = var2;
   }

   private TBSCertificateStructure generateTbsCert() {
      if (!this.extGenerator.isEmpty()) {
         this.tbsGen.setExtensions(this.extGenerator.generate());
      }

      return this.tbsGen.generateTBSCertificate();
   }

   private X509Certificate generateJcaObject(TBSCertificateStructure tbsCert, byte[] signature) {
      ASN1EncodableVector var3;
      ASN1EncodableVector var10000 = var3 = new ASN1EncodableVector;
      var3.<init>();
      var3.add(tbsCert);
      var10000.add(this.sigAlgId);
      var10000.add(new DERBitString(signature));
      return new X509CertificateObject(new X509CertificateStructure(new DERSequence(var3)));
   }

   public void reset() {
      this.tbsGen = new V3TBSCertificateGenerator();
      this.extGenerator.reset();
   }

   public void setSerialNumber(BigInteger serialNumber) {
      if (serialNumber.compareTo(BigInteger.ZERO) > 0) {
         this.tbsGen.setSerialNumber(new DERInteger(serialNumber));
      } else {
         throw new IllegalArgumentException("serial number must be a positive integer");
      }
   }

   public void setIssuerDN(X500Principal issuer) {
      try {
         this.tbsGen.setIssuer(new X509Principal(issuer.getEncoded()));
      } catch (IOException var2) {
         throw new IllegalArgumentException("can't process principal: " + var2);
      }
   }

   public void setIssuerDN(X509Name issuer) {
      this.tbsGen.setIssuer(issuer);
   }

   public void setNotBefore(Date date) {
      this.tbsGen.setStartDate(new Time(date));
   }

   public void setNotAfter(Date date) {
      this.tbsGen.setEndDate(new Time(date));
   }

   public void setSubjectDN(X500Principal subject) {
      try {
         this.tbsGen.setSubject(new X509Principal(subject.getEncoded()));
      } catch (IOException var2) {
         throw new IllegalArgumentException("can't process principal: " + var2);
      }
   }

   public void setSubjectDN(X509Name subject) {
      this.tbsGen.setSubject(subject);
   }

   public void setPublicKey(PublicKey key) {
      try {
         this.tbsGen.setSubjectPublicKeyInfo(SubjectPublicKeyInfo.getInstance((new ASN1InputStream(key.getEncoded())).readObject()));
      } catch (Exception var2) {
         throw new IllegalArgumentException("unable to process key - " + var2.toString());
      }
   }

   public void setSignatureAlgorithm(String signatureAlgorithm) {
      X509V3CertificateGenerator var10000 = this;
      String var10002 = this.signatureAlgorithm = signatureAlgorithm;

      boolean var10001;
      DERObjectIdentifier var6;
      try {
         var6 = X509Util.getAlgorithmOID(var10002);
      } catch (Exception var4) {
         var10001 = false;
         throw new IllegalArgumentException("Unknown signature type requested: " + signatureAlgorithm);
      }

      DERObjectIdentifier var2 = var6;

      String var10003;
      try {
         var10003 = signatureAlgorithm;
         this.sigOID = var2;
      } catch (Exception var3) {
         var10001 = false;
         throw new IllegalArgumentException("Unknown signature type requested: " + signatureAlgorithm);
      }

      AlgorithmIdentifier var5;
      this.sigAlgId = var5 = X509Util.getSigAlgID(var6, var10003);
      var10000.tbsGen.setSignature(var5);
   }

   public void addExtension(String oid, boolean critical, DEREncodable value) {
      X509V3CertificateGenerator var10000 = this;
      DERObjectIdentifier var4;
      var4 = new DERObjectIdentifier.<init>(oid);
      var10000.addExtension(var4, critical, value);
   }

   public void addExtension(DERObjectIdentifier oid, boolean critical, DEREncodable value) {
      this.extGenerator.addExtension(oid, critical, value);
   }

   public void addExtension(String oid, boolean critical, byte[] value) {
      X509V3CertificateGenerator var10000 = this;
      DERObjectIdentifier var4;
      var4 = new DERObjectIdentifier.<init>(oid);
      var10000.addExtension(var4, critical, value);
   }

   public void addExtension(DERObjectIdentifier oid, boolean critical, byte[] value) {
      this.extGenerator.addExtension(oid, critical, value);
   }

   public void copyAndAddExtension(String oid, boolean critical, X509Certificate cert) {
      byte[] cert1;
      if ((cert1 = cert.getExtensionValue(oid)) != null) {
         try {
            this.addExtension((String)oid, critical, (DEREncodable)X509ExtensionUtil.fromExtensionValue(cert1));
         } catch (IOException var4) {
            throw new CertificateParsingException(var4.toString());
         }
      } else {
         throw new CertificateParsingException("extension " + oid + " not present");
      }
   }

   public void copyAndAddExtension(DERObjectIdentifier oid, boolean critical, X509Certificate cert) {
      this.copyAndAddExtension(oid.getId(), critical, cert);
   }

   /** @deprecated */
   public X509Certificate generateX509Certificate(PrivateKey key) {
      try {
         return this.generateX509Certificate(key, "INFOSEC", (SecureRandom)null);
      } catch (NoSuchProviderException var2) {
         throw new SecurityException("BC provider not installed!");
      }
   }

   /** @deprecated */
   public X509Certificate generateX509Certificate(PrivateKey key, SecureRandom random) {
      try {
         return this.generateX509Certificate(key, "INFOSEC", random);
      } catch (NoSuchProviderException var3) {
         throw new SecurityException("BC provider not installed!");
      }
   }

   /** @deprecated */
   public X509Certificate generateX509Certificate(PrivateKey key, String provider) {
      return this.generateX509Certificate(key, provider, (SecureRandom)null);
   }

   /** @deprecated */
   public X509Certificate generateX509Certificate(PrivateKey key, String provider, SecureRandom random) {
      try {
         return this.generate(key, provider, random);
      } catch (NoSuchProviderException var4) {
         throw var4;
      } catch (SignatureException var5) {
         throw var5;
      } catch (InvalidKeyException var6) {
         throw var6;
      } catch (GeneralSecurityException var7) {
         throw new SecurityException("exception: " + var7);
      }
   }

   public X509Certificate generate(PrivateKey key) {
      return this.generate(key, (SecureRandom)null);
   }

   public X509Certificate generate(PrivateKey key, SecureRandom random) {
      X509V3CertificateGenerator var10000 = this;
      TBSCertificateStructure var3;
      TBSCertificateStructure var10001 = var3 = this.generateTbsCert();

      byte[] var10002;
      try {
         var10002 = X509Util.calculateSignature(this.sigOID, this.signatureAlgorithm, key, random, var3);
      } catch (IOException var5) {
         throw new ExtCertificateEncodingException("exception encoding TBS cert", var5);
      }

      try {
         return var10000.generateJcaObject(var10001, var10002);
      } catch (CertificateParsingException var4) {
         throw new ExtCertificateEncodingException("exception producing certificate object", var4);
      }
   }

   public X509Certificate generate(PrivateKey key, String provider) {
      return this.generate(key, provider, (SecureRandom)null);
   }

   public X509Certificate generate(PrivateKey key, String provider, SecureRandom random) {
      X509V3CertificateGenerator var10000 = this;
      TBSCertificateStructure var4;
      TBSCertificateStructure var10001 = var4 = this.generateTbsCert();

      byte[] var10002;
      try {
         var10002 = X509Util.calculateSignature(this.sigOID, this.signatureAlgorithm, provider, key, random, var4);
      } catch (IOException var6) {
         throw new ExtCertificateEncodingException("exception encoding TBS cert", var6);
      }

      try {
         return var10000.generateJcaObject(var10001, var10002);
      } catch (CertificateParsingException var5) {
         throw new ExtCertificateEncodingException("exception producing certificate object", var5);
      }
   }

   public Iterator getSignatureAlgNames() {
      return X509Util.getAlgNames();
   }
}
